#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "globals.h"
#include "functions.h"

/* evaluate the next token read in */

int eval_element(char *newtoken, int tokentype, char *token)
{
char molecule1[LINELENGTH];
char molecule2[LINELENGTH];
char molecule3[LINELENGTH];
char temptoken[TOKENSIZE];
char temp[LINELENGTH];
int t,l;

  strcpy(newtoken,"");
  strcpy(molecule1,"");
  strcpy(molecule2,"");
  strcpy(molecule3,"");

  if (tokentype==2 && strcasecmp(token,"to")==0)
  { return -1; }
    else
  if (tokentype==2 && strcasecmp(token,"and")==0)
  { sprintf(newtoken,"&"); }
    else
  if (tokentype==2 && strcasecmp(token,"xor")==0)
  { sprintf(newtoken,"^"); }
    else
  if (tokentype==2 && strcasecmp(token,"or")==0)
  { sprintf(newtoken,"|"); }
    else
  if (tokentype==2 && strcasecmp(token,"server")==0)
  { parse_server(newtoken); }
    else
/*
  if (tokentype==2 && strcasecmp(token,"session")==0)
  { printf("hellllo!\n"); parse_session_properties(newtoken); }
    else
*/
  if (strcasecmp(token,"instrrev")==0 || strcasecmp(token,"instrrevb")==0)
  {
    tokentype=gettoken(token);
    if (strcasecmp(token,"(")!=0)
    { error(token,"(","InStr"); }
    eval_molecule(molecule1);
    strcpy(molecule2,"");
    strcpy(tokenpush,"(");
    tokentypepush=12;
    eval_funct(molecule2);
    sprintf(newtoken,"(strrpos(%s,%s) ? strrpos(%s,%s)+1 : 0)",molecule1,molecule2,molecule1,molecule2);
  }
    else
  if (strcasecmp(token,"instr")==0 || strcasecmp(token,"instrb")==0)
  {
    tokentype=gettoken(token);
    if (strcasecmp(token,"(")!=0)
    { error(token,"(","InStr"); }
    eval_molecule(molecule1);
    strcpy(molecule2,"");
    strcpy(tokenpush,"(");
    tokentypepush=12;
    eval_funct(molecule2);
    sprintf(newtoken,"(strpos(%s,%s) ? strpos(%s,%s)+1 : 0)",molecule1,molecule2,molecule1,molecule2);
  }
    else
  if (strcasecmp(token,"lcase")==0)
  { sprintf(newtoken,"strtolower("); eval_funct(newtoken); strcat(newtoken,")"); }
    else
  if (strcasecmp(token,"ucase")==0)
  { sprintf(newtoken,"strtoupper("); eval_funct(newtoken); strcat(newtoken,")"); }
    else
  if (strcasecmp(token,"strreverse")==0)
  { sprintf(newtoken,"strrev("); eval_funct(newtoken); strcat(newtoken,")"); }
    else
  if (strcasecmp(token,"len")==0 || strcasecmp(token,"lenb")==0)
  { sprintf(newtoken,"strlen("); eval_funct(newtoken); strcat(newtoken,")"); }
    else
  if (strcasecmp(token,"mid")==0 || strcasecmp(token,"midb")==0)
  {
    tokentype=gettoken(token);
    if (strcasecmp(token,"(")!=0)
    { error(token,"(","Mid"); }

    eval_molecule(molecule1);
    molecule2[strlen(molecule1)]=0;

    t=eval_molecule(molecule2);
    molecule2[strlen(molecule2)]=0;

    if (t==0)
    {
      strcpy(molecule3,"");
      strcpy(tokenpush,"(");

      tokentypepush=12;
      eval_funct(molecule3);
      sprintf(newtoken,"substr(%s,%s-1,%s)",molecule1,molecule2,molecule3);
    }
      else
    {
      sprintf(newtoken,"substr(%s,%s-1)",molecule1,molecule2);
    }
  }
    else
  if (strcasecmp(token,"strcomp")==0)
  {
    tokentype=gettoken(token);
    if (strcasecmp(token,"(")!=0)
    { error(token,"(","StrComp"); }
    eval_molecule(molecule1);
    strcpy(molecule2,"");
    strcpy(tokenpush,"(");
    tokentypepush=12;
    eval_funct(molecule2);
    sprintf(newtoken,"strcmp(%s,%s)",molecule1,molecule2);
  }
    else
  if (strcasecmp(token,"left")==0 || strcasecmp(token,"leftb")==0)
  {
    tokentype=gettoken(token);
    if (strcasecmp(token,"(")!=0)
    { error(token,"(","evalelement: Left"); }
    eval_molecule(molecule1);
    strcpy(molecule2,"");
    strcpy(tokenpush,"(");
    tokentypepush=12;
    eval_funct(molecule2);
    sprintf(newtoken,"substr(%s,0,%s)",molecule1,molecule2);
  }
    else
  if (strcasecmp(token,"right")==0 || strcasecmp(token,"rightb")==0)
  {
    tokentype=gettoken(token);
    if (strcasecmp(token,"(")!=0)
    { error(token,"(","Right"); }
    eval_molecule(molecule1);
    strcpy(molecule2,"");
    strcpy(tokenpush,"(");
    tokentypepush=12;
    eval_funct(molecule2);
    sprintf(newtoken,"substr(%s,strlen(%s)-(%s))",molecule1,molecule1,molecule2);
  }
    else
  if (strcasecmp(token,"replace")==0)
  {
    tokentype=gettoken(token);
/*
    if (strcasecmp(token,"(")!=0)
    { printf(token,"(","Replace"); }
*/

    eval_molecule(molecule1);
    molecule1[strlen(molecule1)]=0;

    eval_molecule(molecule2);
    molecule2[strlen(molecule2)]=0;
/*
    strcpy(molecule3,"");
    push("(",12);
    eval_funct(molecule3);
*/
    eval_molecule(molecule3);

    sprintf(newtoken,"str_replace(%s,%s,%s)",molecule2,molecule3,molecule1);
  }
    else
  if (strcasecmp(token,"session")==0)
  {
    tokentype=gettoken(token);

    if (strcmp(token,".")==0)
    {
      push(token,tokentype);
      parse_session_properties(newtoken);
    }
      else
    if (strcmp(token,"(")==0)
    {
      push(token,tokentype);

      /* sprintf(newtoken,"get_session($sess_dbm,"); */
      eval_funct(newtoken);
      t=session_registered(newtoken);

      if (t!=-1)
      {
        strcpy(temp,sessionpool[t]);
        l=strlen(temp);
        strcpy(temp+l-1,"_session");

        strcpy(newtoken,"$");
        strcat(newtoken,temp+1);
      }
        else
      {
        /* fprintf(out,"session_register(%s);\n",newtoken); */
        strcpy(temp,"${");
        strcat(temp,newtoken);
        strcpy(newtoken,temp);
        strcat(newtoken,".\"_session\"}");
        /* fprintf(out,"${%s.\"_sessin\"}\n",newtoken); */
      }

      /* strcat(newtoken,")"); */
    }
      else
    if (strcmp(token,".")==0)
    {
      tokentype=gettoken(token);
      if (strcasecmp(token,"timeout")==0)
      { strcpy(newtoken,"get_cfg_var(\"session.lifetime\")/60");  }
	else
      { error(token,"?","Session"); }
    }
      else
    { error(token,". (","Session"); }
  }
    else
  if (strcasecmp(token,"empty")==0)
  { sprintf(newtoken,"''"); }
    else
  if (strcasecmp(token,"false")==0 || strcasecmp(token,"vbfalse")==0)
  { sprintf(newtoken,"false"); }
    else
  if (strcasecmp(token,"true")==0 || strcasecmp(token,"vbtrue")==0)
  { sprintf(newtoken,"true"); }
    else
  if (strcasecmp(token,"null")==0 || strcasecmp(token,"nothing")==0)
  { sprintf(newtoken,"null");  }
    else
  if (strcasecmp(token,"array")==0)
  { sprintf(newtoken,"array("); eval_funct(newtoken); strcat(newtoken,")"); }
    else
  if (strcasecmp(token,"lbound")==0)
  { strcpy(newtoken,""); eval_funct(newtoken); strcpy(newtoken,"0"); }
    else
  if (strcasecmp(token,"ubound")==0)
  { sprintf(newtoken,"count("); eval_funct(newtoken); strcat(newtoken,")"); }
    else
  if (strcasecmp(token,"chr")==0)
  { sprintf(newtoken,"chr("); eval_funct(newtoken); strcat(newtoken,")"); }
    else
  if (strcasecmp(token,"asc")==0)
  { sprintf(newtoken,"ord("); eval_funct(newtoken); strcat(newtoken,")"); }
    else
  if (strcasecmp(token,"vartype")==0)
  { sprintf(newtoken,"gettype("); eval_funct(newtoken); strcat(newtoken,")"); }
    else
  if (strcasecmp(token,"isobject")==0)
  { sprintf(newtoken,"is_object("); eval_funct(newtoken); strcat(newtoken,")"); }
    else
  if (strcasecmp(token,"isnumeric")==0)
  { 
    if (php==3)
    { sprintf(newtoken,"IsNumeric("); eval_funct(newtoken); strcat(newtoken,")");  }
      else
    { sprintf(newtoken,"is_numeric("); eval_funct(newtoken); strcat(newtoken,")");  }
 
  }
    else
  if (strcasecmp(token,"isnull")==0)
  { sprintf(newtoken,"!isset("); eval_funct(newtoken); strcat(newtoken,")"); }
    else
  if (strcasecmp(token,"isempty")==0)
  { sprintf(newtoken,"empty("); eval_funct(newtoken); strcat(newtoken,")"); }
    else
  if (strcasecmp(token,"isarray")==0)
  { sprintf(newtoken,"is_array("); eval_funct(newtoken); strcat(newtoken,")"); }
    else
  if (strcasecmp(token,"trim")==0)
  { sprintf(newtoken,"trim("); eval_funct(newtoken); strcat(newtoken,")"); }
    else
  if (strcasecmp(token,"rtrim")==0)
  { sprintf(newtoken,"rtrim("); eval_funct(newtoken); strcat(newtoken,")"); }
    else
  if (strcasecmp(token,"ltrim")==0)
  { sprintf(newtoken,"ltrim("); eval_funct(newtoken); strcat(newtoken,")"); }
    else
  if (strcasecmp(token,"abs")==0)
  { sprintf(newtoken,"Abs("); eval_funct(newtoken); strcat(newtoken,")"); }
    else
  if (strcasecmp(token,"atn")==0)
  { sprintf(newtoken,"Atan("); eval_funct(newtoken); strcat(newtoken,")"); }
    else
  if (strcasecmp(token,"tan")==0)
  { sprintf(newtoken,"Tan("); eval_funct(newtoken); strcat(newtoken,")"); }
    else
  if (strcasecmp(token,"cos")==0)
  { sprintf(newtoken,"Cos("); eval_funct(newtoken); strcat(newtoken,")"); }
    else
  if (strcasecmp(token,"sin")==0)
  { sprintf(newtoken,"Sin("); eval_funct(newtoken); strcat(newtoken,")"); }
    else
  if (strcasecmp(token,"exp")==0)
  { sprintf(newtoken,"Exp("); eval_funct(newtoken); strcat(newtoken,")"); }
    else
  if (strcasecmp(token,"log")==0)
  { sprintf(newtoken,"Log("); eval_funct(newtoken); strcat(newtoken,")"); }
    else
  if (strcasecmp(token,"clng")==0 || strcasecmp(token,"lng")==0)
  { sprintf(newtoken,"intval("); eval_funct(newtoken); strcat(newtoken,")"); }
    else
  if (strcasecmp(token,"cint")==0 || strcasecmp(token,"int")==0)
  { sprintf(newtoken,"intval("); eval_funct(newtoken); strcat(newtoken,")"); }
    else
  if (strcasecmp(token,"cstr")==0)
  { eval_funct(newtoken); }
    else
  if ((strcasecmp(token,"ScriptEngine")==0 || strcasecmp(token,"ScriptEngineMajorVersion")==0) ||
     (strcasecmp(token,"ScriptEngineBuildVersion")==0 || strcasecmp(token,"ScriptEngineMinorVersion")==0))
  {
    tokentype=gettoken(token);
    if (strcmp(token,"(")==0)
    { tokentype=gettoken(token); }
      else
    { push(token,tokentype); }
    sprintf(newtoken,"phpversion()");
  }
    else
  if (strcasecmp(token,"randomize")==0)
  { sprintf(newtoken,"srand("); eval_funct(newtoken); strcat(newtoken,")"); }
    else
  if (strcasecmp(token,"rnd")==0)
  {
    sprintf(newtoken,"(mt_rand(0,10000000)/10000000)");
    tokentype=gettoken(token);
    if (strcmp(token,"(")==0)
    { tokentype=gettoken(token); }
      else
    { push(token,tokentype); }
  }
    else
  if (tokentype==2 && strcasecmp(token,"sqr")==0)
  { sprintf(newtoken,"Sqrt("); eval_funct(newtoken); strcat(newtoken,")"); }
    else
  if (tokentype==2 && strcasecmp(token,"now")==0)
  {
    if (y2k==1)
    { sprintf(newtoken,"strftime(\"%%m/%%d/%%Y %%H:%%M:%%S %%p\")"); }
      else
    { sprintf(newtoken,"strftime(\"%%m/%%d/%%y %%H:%%M:%%S %%p\")"); }
  }
    else
  if (tokentype==2 && strcasecmp(token,"date")==0)
  {
    if (y2k==1)
    { sprintf(newtoken,"strftime(\"%%m/%%d/%%Y\")"); }
      else
    { sprintf(newtoken,"strftime(\"%%m/%%d/%%y\")"); }
  }
    else
  if (tokentype==2 && strcasecmp(token,"year")==0)
  {
    sprintf(newtoken,"strftime(\"%%Y\")");
    eval_funct(newtoken);
  }
    else
  if (tokentype==2 && strcasecmp(token,"month")==0)
  {
    sprintf(newtoken,"strftime(\"%%m\")");
    eval_funct(newtoken);
  }
    else
  if (tokentype==2 && strcasecmp(token,"hour")==0)
  {
    sprintf(newtoken,"strftime(\"%%H\")");
    eval_funct(newtoken);
  }
    else
  if (tokentype==2 && strcasecmp(token,"minute")==0)
  {
    sprintf(newtoken,"strftime(\"%%M\")");
    eval_funct(newtoken);
  }
    else
  if (tokentype==2 && strcasecmp(token,"second")==0)
  {
    sprintf(newtoken,"strftime(\"%%S\")");
    eval_funct(newtoken);
  }
    else
  if (tokentype==2 && strcasecmp(token,"time")==0)
  { sprintf(newtoken,"strftime(\"%%H:%%M:%%S %%p\")"); }
    else
  if (tokentype==2 && strcasecmp(token,"else")==0)
  { 
    fprintf(out,";");
    push2("\r",5);
    push(token,tokentype);

    parse_body(0);
  }
    else
  if (tokentype==2 && strcasecmp(token,"elseif")==0)
  { 
    fprintf(out,";");
    push2("\r",5);
    push(token,tokentype);
    parse_body(0);
  }
    else
  if (tokentype==2 && strcasecmp(token,"end")==0)
  { 
    fprintf(out,";\n");
    indent--;
    autoindent();
    indent++;
    fprintf(out,"}\n");

    gettoken(token);
    // push2("\r",5);
    // push(token,tokentype);

  }
    else
  if (tokentype==2 && strcasecmp(token,"vbcr")==0)
  { sprintf(newtoken,"\"\\r\""); }
    else
  if (tokentype==2 && strcasecmp(token,"vbcrlf")==0)
  { sprintf(newtoken,"\"\\r\\n\""); }
    else
  if (tokentype==2 && strcasecmp(token,"vbnewline")==0)
  { sprintf(newtoken,"\"\\n\""); }
    else
  if (tokentype==2 && strcasecmp(token,"vblf")==0)
  { sprintf(newtoken,"\"\\n\""); }
    else
  if (tokentype==2 && strcasecmp(token,"vbnullchar")==0)
  { sprintf(newtoken,"\"\\0\""); }
    else
  if (tokentype==2 && strcasecmp(token,"vbnullstring")==0)
  { sprintf(newtoken,"NULL"); }
    else
  if (tokentype==2 && strcasecmp(token,"vbtab")==0)
  { sprintf(newtoken,"\"\\t\""); }
    else
  if (tokentype==2 && strcasecmp(token,"request")==0)
  { parse_request(newtoken); }
    else
  if (tokentype==2 && strcasecmp(token,"vbblack")==0)
  { sprintf(newtoken,"0x000000"); }
    else
  if (tokentype==2 && strcasecmp(token,"vbred")==0)
  { sprintf(newtoken,"0xff0000"); }
    else
  if (tokentype==2 && strcasecmp(token,"vbgreen")==0)
  { sprintf(newtoken,"0x00ff00"); }
    else
  if (tokentype==2 && strcasecmp(token,"vbyellow")==0)
  { sprintf(newtoken,"0xffff00"); }
    else
  if (tokentype==2 && strcasecmp(token,"vbblue")==0)
  { sprintf(newtoken,"0x0000ff"); }
    else
  if (tokentype==2 && strcasecmp(token,"vbmagenta")==0)
  { sprintf(newtoken,"0xff00ff"); }
    else
  if (tokentype==2 && strcasecmp(token,"vbcyan")==0)
  { sprintf(newtoken,"0xffff00"); }
    else
  if (tokentype==2 && strcasecmp(token,"vbwhite")==0)
  { sprintf(newtoken,"0xffffff"); }
    else
  if (tokentype==2 && strcasecmp(token,"set")==0)
  { parse_set(newtoken); }
    else
  if (tokentype==9)
  { sprintf(newtoken,"."); }
    else
  if (tokentype==14)
  { sprintf(newtoken,"%s",token); }
    else
  if (tokentype==10 || tokentype==11)
  { sprintf(newtoken,"%s",token); }
    else
  if ((tokentype==7 || tokentype==12) ||
        (tokentype==8 || tokentype==1))
  { sprintf(newtoken,"%s",token); }
    else
  if (tokentype==2 && ((t=getobject(token))!=0))
  {
    strcpy(temptoken,token);
    tokentype=gettoken(token);
    if (strcmp(token,".")!=0)
    { push(token,tokentype); }

    if (t==1) 
    { 
      if (database_type==0)
      { parse_adodb_connection_mysql(newtoken,temptoken); }
        else
      if (database_type==1)
      { parse_adodb_connection_odbc(newtoken,temptoken); }
        else
      if (database_type==2)
      { parse_adodb_connection_oracle(newtoken,temptoken); }
        else
      if (database_type==3)
      { parse_adodb_connection_sybase(newtoken,temptoken); }
        else
      if (database_type==4)
      { parse_adodb_connection_pg(newtoken,temptoken); }
    }
      else
    if (t==2) 
    { 
      if (database_type==0)
      { parse_adodb_rs_mysql(newtoken,temptoken);  }
        else
      if (database_type==1)
      { parse_adodb_rs_odbc(newtoken,temptoken);  }
        else
      if (database_type==2)
      { parse_adodb_rs_oracle(newtoken,temptoken);  }
        else
      if (database_type==3)
      { parse_adodb_rs_sybase(newtoken,temptoken);  }
        else
      if (database_type==4)
      { parse_adodb_rs_pg(newtoken,temptoken);  }
    }
      else
    if (t==3) { parse_command(newtoken,temptoken); }
      else
    if (t==5) { parse_filesystem(newtoken,temptoken); }
      else
    if (t==6) { parse_mail(newtoken,temptoken); }
      else
    if (t==7) { parse_new_mail(newtoken,temptoken); }
  }
    else
  if (isfunction(token)==1)
  {
    sprintf(newtoken,"%s",token);
    t=gettoken(token);
    if (strcmp(token,"(")==0)
    {
      strcat(newtoken,"(");
      push(token,tokentype);
/*
      strcpy(tokenpush,"(");
      tokentypepush=tokentype;
*/
      eval_funct(newtoken);
      strcat(newtoken,")");
    }
      else
    { 
   /* error(token,"(","eval_element [function]");  */
      strcat(newtoken,"()");
    }
  }
    else
  if (tokentype==2)
  {
    strcase(token);
    sprintf(newtoken,"$%s",token);
    tokentype=gettoken(token);
    push(token,tokentype);
    
    if (strcmp(token,"(")==0)
    {
      eval_funct(molecule1);
      sprintf(molecule2,"[%s]",molecule1);
      strcat(newtoken,molecule2);
    }
  }

  if (spacer==1) strcat(newtoken," ");

  return 0;
}

